@@ -67,7 +67,6 @@ from utils import get_version
 from FunkLoadHTTPServer import FunkLoadHTTPRequestHandler
 from FunkLoadHTTPServer import FunkLoadHTTPServer
 
-
 # ------------------------------------------------------------
 # utils
 #
@@ -119,7 +118,7 @@ def reset_cycle_results():
 
 
 def load_unittest(test_module, test_class, test_name, options):
-    """Instantiate a unittest."""
+    """instantiate a unittest."""
     module = __import__(test_module)
     klass = getattr(module, test_class)
     return klass(test_name, options)
@@ -239,10 +238,11 @@ class BenchRunner:
 
         # setup monitoring
         monitor_hosts = []                  # list of (host, port, descr)
-        for host in test.conf_get('monitor', 'hosts', '', quiet=True).split():
-            host = host.strip()
-            monitor_hosts.append((host, test.conf_getInt(host, 'port'),
-                                  test.conf_get(host, 'description', '')))
+        if not options.is_distributed:
+            for host in test.conf_get('monitor', 'hosts', '', quiet=True).split():
+                host = host.strip()
+                monitor_hosts.append((host, test.conf_getInt(host, 'port'),
+                                      test.conf_get(host, 'description', '')))
         self.monitor_hosts = monitor_hosts
         # keep the test to use the result logger for monitoring
         # and call setUp/tearDown Cycle
@@ -259,6 +259,9 @@ class BenchRunner:
         cycle = total_success = total_failures = total_errors = 0
 
         self.logr_open()
+        trace("* setUpBench hook: ...")
+        self.test.setUpBench()
+        trace(' done.\n\n')
         for cvus in self.cycles:
             t_start = time.time()
             reset_cycle_results()
@@ -266,7 +269,9 @@ class BenchRunner:
             trace(text)
             trace('-' * (len(text) - 1) + "\n\n")
             monitor_key = '%s:%s:%s' % (self.method_name, cycle, cvus)
+            trace("* setUpCycle hook: ...")
             self.test.setUpCycle()
+            trace(' done.\n')
             self.startMonitors(monitor_key)
             self.startThreads(cycle, cvus)
             self.logging()
@@ -274,7 +279,9 @@ class BenchRunner:
             self.stopThreads()
             self.stopMonitors(monitor_key)
             cycle += 1
+            trace("* tearDownCycle hook: ...")
             self.test.tearDownCycle()
+            trace(' done.\n')
             t_stop = time.time()
             trace("* End of cycle, %.2fs elapsed.\n" % (t_stop - t_start))
             success, failures, errors = get_cycle_results()
@@ -285,6 +292,9 @@ class BenchRunner:
             total_success += success
             total_failures += failures
             total_errors += errors
+        trace("* tearDownBench hook: ...")
+        self.test.tearDownBench()
+        trace(' done.\n\n')
         self.logr_close()
 
         # display bench result
@@ -578,6 +588,7 @@ def main():
                       help="Do not fail if css/image links are "
                       "not reachable.")
     parser.add_option("", "--simple-fetch", action="store_true",
+                      dest="bench_simple_fetch",
                       help="Don't load additional links like css "
                       "or images when fetching an html page.")
     parser.add_option("-l", "--label", type="string",
@@ -593,7 +604,20 @@ def main():
                       "/getcvu returns number of CVUs ")
     parser.add_option("", "--debug-server-port", type="string", dest="debugport",
                       help="Port at which debug server should run during the test")
+    
+    parser.add_option("","--distribute", action="store_true", dest="distribute",
+                      help="Distributes the CVUs over a group of worker machines "
+                      "that are defined in the section [workers]")
+    parser.add_option("","--distribute-workers", type="string", dest="workerlist",
+                      help="This parameter will  over-ride the list of workers defined "
+                      "in the config file. expected notation is uname@host,uname:pwd@host or just host...") 
+    parser.add_option("","--is-distributed", action="store_true", dest="is_distributed",
+                      help="This parameter is for internal use only. it signals to a "
+                      "worker node that it is in distributed mode and shouldn't "
+                      "perform certain actions.")
+
     options, args = parser.parse_args()
+    cmd_args = " ".join([k for k in sys.argv[1:] if k.find('--distribute')<0])
     if len(args) != 2:
         parser.error("incorrect number of arguments")
     if not args[1].count('.'):
@@ -603,19 +627,34 @@ def main():
         options.bench_sleep_time_max = '0'
         options.bench_sleep_time = '0'
     klass, method = args[1].split('.')
-    bench = BenchRunner(args[0], klass, method, options)
+    if options.distribute:
+        from Distributed import DistributionMgr
+        ret = None
+        try:
+            distmgr = DistributionMgr( args[0] , klass, method, options, cmd_args )     
+            try:
+                distmgr.prepare_workers(allow_errors = True)
+                ret = distmgr.run()
+                distmgr.final_collect()
+            except KeyboardInterrupt:
+                trace("* ^C received *")
+                distmgr.abort()
+        except UserWarning,error:
+            trace(red_str("Distribution failed with:%s \n" % (error)))
+        sys.exit(ret)
+    else:
+        bench = BenchRunner(args[0], klass, method, options)
+        # Start a HTTP server optionally
+        if options.debugserver == True:
+            http_server_thread = FunkLoadHTTPServer(bench, options.debugport)
+            http_server_thread.start()
 
-    # Start a HTTP server optionally
-    if options.debugserver == True:
-        http_server_thread = FunkLoadHTTPServer(bench, options.debugport)
-        http_server_thread.start()
-
-    ret = None
-    try:
-        ret = bench.run()
-    except KeyboardInterrupt:
-        trace("* ^C received *")
-    sys.exit(ret)
+        ret = None
+        try:
+            ret = bench.run()
+        except KeyboardInterrupt:
+            trace("* ^C received *")
+        sys.exit(ret)
 
 if __name__ == '__main__':
     main()
