@@ -17,7 +17,7 @@
 #
 """Create an ReST or HTML report with charts from a FunkLoad bench xml result.
 
-Producing html and png chart require python-docutils and python-gdchart
+Producing html and png chart require python-docutils and gnuplot
 
 $Id: ReportBuilder.py 24737 2005-08-31 09:00:16Z bdelbosc $
 """
@@ -40,10 +40,12 @@ Examples
   %prog --html -o /tmp funkload.xml
                         Build an HTML report in /tmp
   %prog --html node1.xml node2.xml node3.xml
-                        Build an HTML report merging test result from 3 nodes.
-  %prog --diff /tmp/test_reader-20080101 /tmp/test_reader-20080102
+                        Build an HTML report merging test results from 3 nodes.
+  %prog --diff /path/to/report-reference /path/to/report-challenger
                         Build a differential report to compare 2 bench reports,
                         requires gnuplot.
+  %prog --trend /path/to/report-dir1 /path/to/report-1 ... /path/to/report-n
+                        Build a trend report using multiple reports.
   %prog -h
                         More options.
 """
@@ -62,6 +64,7 @@ from ReportStats import MonitorStat, ErrorStat
 from ReportRenderRst import RenderRst
 from ReportRenderHtml import RenderHtml
 from ReportRenderDiff import RenderDiff
+from ReportRenderTrend import RenderTrend
 from MergeResultFiles import MergeResultFiles
 from utils import trace, get_version
 
@@ -210,6 +213,8 @@ def main():
                           version="FunkLoad %s" % get_version())
     parser.add_option("-H", "--html", action="store_true", default=False,
                       dest="html", help="Produce an html report.")
+    parser.add_option("--org", action="store_true", default=False,
+                      dest="org", help="Org-mode report.")
     parser.add_option("-P", "--with-percentiles", action="store_true",
                       default=True, dest="with_percentiles",
                       help=("Include percentiles in tables, use 10%, 50% and"
@@ -217,11 +222,14 @@ def main():
     parser.add_option("--no-percentiles", action="store_false",
                       dest="with_percentiles",
                       help=("No percentiles in tables display min, "
-                            "avg and max in charts (gdchart only)."))
+                            "avg and max in charts."))
     cur_path = os.path.abspath(os.path.curdir)
     parser.add_option("-d", "--diff", action="store_true",
                       default=False, dest="diffreport",
                       help=("Create differential report."))
+    parser.add_option("-t", "--trend", action="store_true",
+                      default=False, dest="trendreport",
+                      help=("Build a trend reprot."))
     parser.add_option("-o", "--output-directory", type="string",
                       dest="output_dir",
                       help="Parent directory to store reports, the directory"
@@ -246,6 +254,14 @@ def main():
         html_path = RenderDiff(args[0], args[1], options)
         trace("done: \n")
         trace("%s\n" % html_path)
+    elif options.trendreport:
+        if len(args) < 2:
+            parser.error("incorrect number of arguments")
+        trace("Creating trend report ... ")
+        output_dir = options.output_dir
+        html_path = RenderTrend(args, options)
+        trace("done: \n")
+        trace("%s\n" % html_path)
     else:
         if len(args) < 1:
             parser.error("incorrect number of arguments")
@@ -267,6 +283,11 @@ def main():
                                    options)()
             trace("done: \n")
             trace(html_path + "\n")
+        elif options.org:
+            from ReportRenderOrg import RenderOrg
+            print str(RenderOrg(xml_parser.config, xml_parser.stats,
+                                xml_parser.error, xml_parser.monitor,
+                                options))
         else:
             print str(RenderRst(xml_parser.config, xml_parser.stats,
                                 xml_parser.error, xml_parser.monitor,
