@@ -17,21 +17,14 @@
 #
 """Choose the best html rendering
 
-$Id: ReportRenderHtml.py 54287 2011-02-04 23:40:50Z bdelbosc $
+$Id$
 """
 
 try:
     # 1/ gnuplot
     from ReportRenderHtmlGnuPlot import RenderHtmlGnuPlot as RenderHtml
 except ImportError:
-    try:
-        # 2/ gdchart2
-        from ReportRenderHtmlGDChart import RenderHtmlGDChart as RenderHtml
-        import warnings
-        warnings.warn('GDChart library is deprecated and will be removed '
-                      'in 1.15, FunkLoad is now using gnuplot > 4.2')
-    except:
-        # 3/ no charts
-        from ReportRenderHtmlBase import RenderHtmlBase as RenderHtml
+    # 2/ no charts
+    from ReportRenderHtmlBase import RenderHtmlBase as RenderHtml
 
 from ReportRenderHtmlGnuPlot import RenderHtmlGnuPlot as RenderHtml
