@@ -17,16 +17,17 @@
 #
 """Classes that render statistics.
 
-$Id: ReportRenderRst.py 54265 2011-01-11 23:21:46Z bdelbosc $
+$Id$
 """
 import os
 from shutil import copyfile
 from utils import get_version
 
+LI = '*'
 # ------------------------------------------------------------
 # ReST rendering
 #
-def rst_title(title, level=1):
+def rst_title(title, level=1, newpage=True):
     """Return a rst title."""
     rst_level = ['=', '=', '-', '~']
     if level == 0:
@@ -57,6 +58,7 @@ class BaseRst:
     fmt_str = "%18s"
     fmt_percent = "%16.2f%%"
     fmt_deco = "=================="
+    sep = " "
     headers = []
     indent = 0
     image_names = []
@@ -158,20 +160,20 @@ class AllResponseRst(BaseRst):
         ret.append(self.fmt_float % stats.max)
         if self.with_percentiles:
             self._attach_percentiles(ret)
-        ret = ' '.join(ret)
+        ret = self.sep.join(ret)
         return ret
 
 
 class PageRst(AllResponseRst):
     """Page rendering."""
-    headers = ["CUs", "Apdex*", "Rating*", "SPPS", "maxSPPS", "TOTAL", "SUCCESS",
+    headers = ["CUs", "Apdex*", "Rating", "SPPS", "maxSPPS", "TOTAL", "SUCCESS",
               "ERROR", "MIN", "AVG", "MAX"]
     image_names = ['pages_spps', 'pages']
     with_apdex = True
 
 class ResponseRst(BaseRst):
     """Response rendering."""
-    headers = ["CUs", "Apdex*", "Rating*", "TOTAL", "SUCCESS", "ERROR", "MIN", "AVG", "MAX"]
+    headers = ["CUs", "Apdex*", "Rating", "TOTAL", "SUCCESS", "ERROR", "MIN", "AVG", "MAX"]
     indent = 4
     image_names = ['request_']
     with_apdex = True
@@ -198,7 +200,7 @@ class ResponseRst(BaseRst):
         ret.append(self.fmt_float % stats.max)
         if self.with_percentiles:
             self._attach_percentiles(ret)
-        ret = ' '.join(ret)
+        ret = self.sep.join(ret)
         return ret
 
 
@@ -218,7 +220,7 @@ class TestRst(BaseRst):
         ret.append(self.fmt_int % stats.count)
         ret.append(self.fmt_int % stats.success)
         ret.append(self.fmt_percent % stats.error_percent)
-        ret = ' '.join(ret)
+        ret = self.sep.join(ret)
         return ret
 
 
@@ -244,6 +246,8 @@ class RenderRst:
             self.with_chart = True
         else:
             self.with_chart = False
+        self.date = config['time'][:19].replace('T', ' ')
+
 
     def getRepresentativeCycleStat(self):
         """Return the cycle stat with the maximum number of steps."""
@@ -299,13 +303,11 @@ class RenderRst:
         """Append text to rst output."""
         self.rst.append(text)
 
-    def renderConfig(self):
-        """Render bench configuration and metadata."""
+    def renderHeader(self):
         config = self.config
         self.append(rst_title("FunkLoad_ bench report", 0))
         self.append('')
-        date = config['time'][:19].replace('T', ' ')
-        self.append(':date: ' + date)
+        self.append(':date: ' + self.date)
         description = [config['class_description']]
         description += ["Bench result of ``%s.%s``: " % (config['class'],
                                                        config['method'])]
@@ -318,26 +320,30 @@ class RenderRst:
         self.append(".. contents:: Table of contents")
         self.append(".. |APDEXT| replace:: \ :sub:`%.1f`" % self.options.apdex_t)
 
+    def renderConfig(self):
+        """Render bench configuration and metadata."""
+        self.renderHeader()
+        config = self.config
         self.append(rst_title("Bench configuration", 2))
-        self.append("* Launched: %s" % date)
+        self.append(LI + " Launched: %s" % self.date)
         if config.get('node'):
-            self.append("* From: %s" % config['node'])
-        self.append("* Test: ``%s.py %s.%s``" % (config['module'],
+            self.append(LI + " From: %s" % config['node'])
+        self.append(LI + " Test: ``%s.py %s.%s``" % (config['module'],
                                                  config['class'],
                                                  config['method']))
         if config.get('label'):
-            self.append("* Label: %s" % config['label'])
-        self.append("* Target server: %s" % config['server_url'])
-        self.append("* Cycles of concurrent users: %s" % config['cycles'])
-        self.append("* Cycle duration: %ss" % config['duration'])
-        self.append("* Sleeptime between request: from %ss to %ss" % (
+            self.append(LI + " Label: %s" % config['label'])
+        self.append(LI + " Target server: %s" % config['server_url'])
+        self.append(LI + " Cycles of concurrent users: %s" % config['cycles'])
+        self.append(LI + " Cycle duration: %ss" % config['duration'])
+        self.append(LI + " Sleeptime between request: from %ss to %ss" % (
             config['sleep_time_min'], config['sleep_time_max']))
-        self.append("* Sleeptime between test case: %ss" %
+        self.append(LI + " Sleeptime between test case: %ss" %
                     config['sleep_time'])
-        self.append("* Startup delay between thread: %ss" %
+        self.append(LI + " Startup delay between thread: %ss" %
                     config['startup_delay'])
-        self.append("* Apdex |APDEXT|")
-        self.append("* FunkLoad_ version: %s" % config['version'])
+        self.append(LI + " Apdex: |APDEXT|")
+        self.append(LI + " FunkLoad_ version: %s" % config['version'])
         self.append("")
         # check for metadata
         has_meta = False
@@ -347,22 +353,22 @@ class RenderRst:
                     self.append("Bench metadata:")
                     self.append('')
                     has_meta = True
-                self.append("* %s: %s" % (key[5:], config[key]))
+                self.append(LI + " %s: %s" % (key[5:], config[key]))
         if has_meta:
             self.append("")
 
     def renderTestContent(self, test):
         """Render global information about test content."""
-        self.append(rst_title("Bench content", 2))
+        self.append(rst_title("Bench content", 2)) 
         config = self.config
         self.append('The test ``%s.%s`` contains: ' % (config['class'],
                                                        config['method']))
         self.append('')
-        self.append("* %s page(s)" % test.pages)
-        self.append("* %s redirect(s)" % test.redirects)
-        self.append("* %s link(s)" % test.links)
-        self.append("* %s image(s)" % test.images)
-        self.append("* %s XML RPC call(s)" % test.xmlrpc)
+        self.append(LI + " %s page(s)" % test.pages)
+        self.append(LI + " %s redirect(s)" % test.redirects)
+        self.append(LI + " %s link(s)" % test.links)
+        self.append(LI + " %s image(s)" % test.images)
+        self.append(LI + " %s XML RPC call(s)" % test.xmlrpc)
         self.append('')
 
         self.append('The bench contains:')
@@ -386,11 +392,11 @@ class RenderRst:
                 total_responses += stats[cycle]['response'].count
                 total_responses_error += stats[cycle]['response'].error
         self.append('')
-        self.append("* %s tests" % total_tests + (
+        self.append(LI + " %s tests" % total_tests + (
             total_tests_error and ", %s error(s)" % total_tests_error or ''))
-        self.append("* %s pages" % total_pages + (
+        self.append(LI + " %s pages" % total_pages + (
             total_pages_error and ", %s error(s)" % total_pages_error or ''))
-        self.append("* %s requests" % total_responses + (
+        self.append(LI + " %s requests" % total_responses + (
             total_responses_error and ", %s error(s)" %
             total_responses_error or ''))
         self.append('')
@@ -451,14 +457,17 @@ class RenderRst:
         steps.sort()
         self.steps = steps
         current_step = -1
+        newpage = False
         for step_name in steps:
             a_step = cycle_r_steps[step_name]
             if a_step.step != current_step:
                 current_step = a_step.step
                 self.append(rst_title("PAGE %s: %s" % (
-                    a_step.step, a_step.description or a_step.url), 3))
-            self.append('* Req: %s, %s, url %s' % (a_step.number,
-                                                   a_step.type, a_step.url))
+                    a_step.step, a_step.description or a_step.url), 3, newpage))
+                newpage = True
+            self.append(LI + ' Req: %s, %s, url ``%s``' % (a_step.number,
+                                                           a_step.type,
+                                                           a_step.url))
             self.append('')
             self.renderCyclesStepStat(step_name)
 
@@ -481,7 +490,7 @@ class RenderRst:
     def renderSlowestRequests(self, number):
         """Render the n slowest requests of the best cycle."""
         stats = self.stats
-        self.append(rst_title("%i Slowest requests"% number, 2))
+        self.append(rst_title("Slowest requests", 2))
         cycle = self.getBestCycle()
         cycle_name = None
         if not (cycle and stats[cycle].has_key('response_step')):
@@ -499,10 +508,10 @@ class RenderRst:
 
         items.sort()
         items.reverse()
-        self.append('Slowest average response time during the best cycle '
-                    'with **%s** CUs:\n' % cycle_name)
+        self.append('The %d slowest average response time during the '
+                    'best cycle with **%s** CUs:\n' % (number, cycle_name))
         for item in items[:number]:
-            self.append('* In page %s, Apdex rating: %s, avg response time: %3.2fs, %s: %s\n'
+            self.append(LI + ' In page %s, Apdex rating: %s, avg response time: %3.2fs, %s: ``%s``\n'
                         '  `%s`' % (
                 item[1], get_apdex_label(item[5]), item[0], item[2], item[3], item[4]))
 
@@ -530,7 +539,7 @@ class RenderRst:
             for err_type in err_types:
                 stat = errors[err_type][0]
                 if err_type[1]:
-                    self.append('* %s time(s), code: %s, %s\n'
+                    self.append(LI + ' %s time(s), code: %s, %s\n'
                                 '  in %s, line %s: %s' %(
                         len(errors[err_type]),
                         err_type[0],
@@ -540,7 +549,7 @@ class RenderRst:
                 else:
                     traceback = stat.traceback and stat.traceback.replace(
                         'File ', '\n    File ') or 'No traceback.'
-                    self.append('* %s time(s), code: %s::\n\n'
+                    self.append(LI + ' %s time(s), code: %s::\n\n'
                                 '    %s\n' %(
                         len(errors[err_type]),
                         err_type[0], traceback))
@@ -548,50 +557,52 @@ class RenderRst:
     def renderDefinitions(self):
         """Render field definition."""
         self.append(rst_title("Definitions", 2))
-        self.append('* CUs: Concurrent users or number of concurrent threads'
+        self.append(LI + ' CUs: Concurrent users or number of concurrent threads'
                     ' executing tests.')
-        self.append('* Request: a single GET/POST/redirect/xmlrpc request.')
-        self.append('* Page: a request with redirects and resource'
+        self.append(LI + ' Request: a single GET/POST/redirect/xmlrpc request.')
+        self.append(LI + ' Page: a request with redirects and resource'
                     ' links (image, css, js) for an html page.')
-        self.append('* STPS: Successful tests per second.')
-        self.append('* SPPS: Successful pages per second.')
-        self.append('* RPS: Requests per second, successful or not.')
-        self.append('* maxSPPS: Maximum SPPS during the cycle.')
-        self.append('* maxRPS: Maximum RPS during the cycle.')
-        self.append('* MIN: Minimum response time for a page or request.')
-        self.append('* AVG: Average response time for a page or request.')
-        self.append('* MAX: Maximmum response time for a page or request.')
-        self.append('* P10: 10th percentile, response time where 10 percent'
+        self.append(LI + ' STPS: Successful tests per second.')
+        self.append(LI + ' SPPS: Successful pages per second.')
+        self.append(LI + ' RPS: Requests per second, successful or not.')
+        self.append(LI + ' maxSPPS: Maximum SPPS during the cycle.')
+        self.append(LI + ' maxRPS: Maximum RPS during the cycle.')
+        self.append(LI + ' MIN: Minimum response time for a page or request.')
+        self.append(LI + ' AVG: Average response time for a page or request.')
+        self.append(LI + ' MAX: Maximmum response time for a page or request.')
+        self.append(LI + ' P10: 10th percentile, response time where 10 percent'
                     ' of pages or requests are delivered.')
-        self.append('* MED: Median or 50th percentile, response time where half'
+        self.append(LI + ' MED: Median or 50th percentile, response time where half'
                     ' of pages or requests are delivered.')
-        self.append('* P90: 90th percentile, response time where 90 percent'
+        self.append(LI + ' P90: 90th percentile, response time where 90 percent'
                     ' of pages or requests are delivered.')
-        self.append('* P95: 95th percentile, response time where 95 percent'
+        self.append(LI + ' P95: 95th percentile, response time where 95 percent'
                     ' of pages or requests are delivered.')
-        self.append('''* Apdex: Application Performance Index, 
+        self.append(LI + ''' Apdex T: Application Performance Index, 
   this is a numerical measure of user satisfaction, it is based
   on three zones of application responsiveness:
 
-    - Satisfied: The user is fully productive. This represents the
-      time value (T seconds) below which users are not impeded by
-      application response time.
+  - Satisfied: The user is fully productive. This represents the
+    time value (T seconds) below which users are not impeded by
+    application response time.
 
-    - Tolerating: The user notices performance lagging within
-      responses greater than T, but continues the process.
+  - Tolerating: The user notices performance lagging within
+    responses greater than T, but continues the process.
 
-    - Frustrated: Performance with a response time greater than 4*T
-      seconds is unacceptable, and users may abandon the process.
+  - Frustrated: Performance with a response time greater than 4*T
+    seconds is unacceptable, and users may abandon the process.
 
-  By default T is set to 1.5s this means that response time between 0
-  and 1.5s the user is fully productive, between 1.5 and 6s the
-  responsivness is tolerating and above 6s the user is frustrated.
+    By default T is set to 1.5s this means that response time between 0
+    and 1.5s the user is fully productive, between 1.5 and 6s the
+    responsivness is tolerating and above 6s the user is frustrated.
 
-  The Apdex score converts many measurements into one number on a
-  uniform scale of 0-to-1 (0 = no users satisfied, 1 = all users
-  satisfied).
+    The Apdex score converts many measurements into one number on a
+    uniform scale of 0-to-1 (0 = no users satisfied, 1 = all users
+    satisfied).
 
-  To ease interpretation the Apdex score is also represented as a rating:
+    Visit http://www.apdex.org/ for more information.''')
+        self.append(LI + ''' Rating: To ease interpretation the Apdex
+  score is also represented as a rating:
 
   - U for UNACCEPTABLE represented in gray for a score between 0 and 0.5 
 
@@ -601,14 +612,16 @@ class RenderRst:
 
   - G for Good represented in green for a score between 0.85 and 0.94
 
-  - E for Excellent represented in blue for a score between 0.94 and 1
-
-  visit http://www.apdex.org/ for more information.''')
+  - E for Excellent represented in blue for a score between 0.94 and 1.''')
         self.append('')
         self.append('Report generated with FunkLoad_ ' + get_version() +
                     ', more information available on the '
                     '`FunkLoad site <http://funkload.nuxeo.org/#benching>`_.')
 
+    def renderHook(self):
+        """Hook for post processing"""
+        pass
+
     def __repr__(self):
         self.renderConfig()
         if not self.cycles:
@@ -634,6 +647,7 @@ class RenderRst:
         self.renderPageDetail(cycle_r)
         self.renderErrors()
         self.renderDefinitions()
+        self.renderHook()
         return '\n'.join(self.rst)
 
 
